local blocks = require("blocks.blocks")
local Block = require("blocks.Block")
local points = require("point")
local deltas = points.deltas
local Point = points.Point
local blockBuilding = require("blocks.blockBuilding")

local FlowerStem  = Block:new()
FlowerStem.maxDurability = 10
FlowerStem.mass = 0.2
FlowerStem.transparent = true

function FlowerStem:new()
  self.__index = self
  local instance = Block.new(FlowerStem)
  setmetatable(instance,self)  
  instance.superClass = Block
  blockBuilding.makeTicking(instance,100)
  instance:posCheck()
  return instance
end

function FlowerStem:tick()
  --print("FLOWER TICKING "..tostring(self.position))
  local level = self.level
  if not level then return end
  
  local planned = false
  for _, delta in pairs(deltas) do
    local nearBlockPosition = self.position + delta
    local nearBlock = level:getBlock(nearBlockPosition.x, nearBlockPosition.y)
    --print("near flower is "..nearBlock.blockId)
    if nearBlock.blockId == "core:sky" then
      --print("FLOWER PLANNING")
      local newBlockId =  math.random() > 0.4 and "core:flowerStem" or "core:flower"
      
      local function makePutter(newId, newPosition)
        local newBlockId = newId
        local position = newPosition:clone()
        local function putBlock()
          local newBlock = blocks.new(newBlockId)
          newBlock:posCheck()
          level:setBlock(position.x, position.y, newBlock)
        end
        return putBlock        
      end
      
      local putBlock = makePutter(newBlockId, nearBlockPosition)
      dispatcher:runOnce("enterFrame", putBlock)      
      planned = true
      break
    end
  end
  if not planned then
    self.turnHelper.interval = 10000
  else
    self.turnHelper.interval = 100
  end
end

return FlowerStem