local Block = require("blocks.Block")
local point = require("point")
local blocks = require("blocks.blocks")


local Water = Block:new()
Water.transparent = true
Water.passable = true
Water.isLiquid = true
Water.pressureThreshold = 0.01
Water.maxCalmPressure = 10
Water.excess = 1
Water.overPressureTop = 5

function Water:new()
  self.__index = self
  local instance = Block.new(Water)
  setmetatable(instance, self)
  instance.pressure = 10
  
  return instance
end

function Water:pressureAround(x,y)
  if not self.level then
    return
  end
  local level = self.level  
  local result = 0
  local count = 0
  local air = 0
  for dx=-1,1 do
    for dy = -1,1 do
      -- if (dx~=0)~=(dy~=0) then            
      --[[if dx~=0 or dy~=0 then
        local neighbour = level:getBlock(x+dx,y+dy)
        local neighbourPressure = neighbour.pressure and neighbour.pressure - dy or 1
        if neighbourPressure > result then
          --print("np "..tostring(neighbour.position).."="..neighbourPressure)
          result = neighbourPressure
        end
      end 
      ]]--
      if (dx~=0)~=(dy~=0) then 
        local neighbour = level:getBlock(x+dx,y+dy)
        
       
        local pressure = neighbour.pressure
        if not pressure and neighbour.isReplacible then
         -- pressure = 1
          air = air + 1
        end
        if pressure then
          result = result + pressure - dy
          count = count + 1
        end
      end
    end
  end
  return  count > 1 and math.max(1,result / count) or 1 
end

function Water:updatePressure()
  error('Not expected to be called!')
  if not self.level then
    return false
  end
  local level = self.level
  local x = self.position.x
  local y = self.position.y
  local result = self:pressureAround(x,y)
  
  if math.abs(self.pressure - result)> self.pressureThreshold then
    self.pressure = result
    level:scheduleNeighboursForUpdate(self)
    return true
  end
  return false
  
end

function Water:tick(chain)
  local level = self.level
  if not level then
    print("NOT LEVEL")
    return
  end
  level.liquidHelper:handleBlock(self)
  --[[local pos = self.position:clone()
  
  local function handleSideBlock(sideBlock, moveAll)
    
    local otherBlock =  level:getBlockPt(self.position - (sideBlock.position - self.position))
    local nextBlock = level:getBlockPt(sideBlock.position + (sideBlock.position - self.position))
    local extraSideMove = self.position.y == sideBlock.position.y and  otherBlock.isLiquid and  otherBlock.pressure > self.pressure and (nextBlock.isReplacible or nextBlock.isLiquid and nextBlock.pressure < otherBlock.pressure)
    
    if moveAll and sideBlock.isReplacible then
      level:setBlockPt(sideBlock.position, self)
      level:setBlockPt(pos, sideBlock)
      
      level:scheduleNeighboursForUpdate(self)
      level:scheduleNeighboursForUpdate(sideBlock)
      return true
    end
    if sideBlock.isReplacible and (self.pressure > 1 or extraSideMove)  then
      local movedPart = 1--math.max(1,math.floor(self.pressure*0.25))
      local newLiquid = self.__index:new()
      newLiquid.pressure = movedPart
      level:setBlockPt(sideBlock.position, newLiquid)
      self.pressure = self.pressure - movedPart
      if self.pressure <= 0 then
         level:setBlockPt(pos, blocks.new("core:sky"))   
      end
      level:scheduleNeighboursForUpdate(self)
      level:scheduleNeighboursForUpdate(newLiquid)
      return true
    end
    if sideBlock.isLiquid then
      local maxOverPressure = math.max(self.pressure, sideBlock.maxCalmPressure)+sideBlock.excess
      local maxThisOverPressure = math.max(sideBlock.pressure, self.maxCalmPressure)+self.excess
      local canUseOverPressure = moveAll and self.pressure > self.overPressureTop and sideBlock.pressure < maxOverPressure
      local moveOverPressure = math.min(self.pressure - self.overPressureTop, maxOverPressure - sideBlock.pressure)
      local upDirection = sideBlock.position.y < self.position.y
      local moveUpPossible = sideBlock.pressure < sideBlock.overPressureTop and self.pressure > self.maxCalmPressure or self.pressure > maxThisOverPressure
      local wrongMove = upDirection and not moveUpPossible
      

      if not wrongMove and (self.pressure  > sideBlock.pressure +1 or extraSideMove and self.pressure >= sideBlock.pressure    or  canUseOverPressure) then
        local moveAmount
        if canUseOverPressure then
          moveAmount = moveOverPressure
        elseif moveAll then
          moveAmount = math.min(self.pressure, sideBlock.maxCalmPressure - sideBlock.pressure)
        else
          moveAmount = 1 --math.max(1,math.floor(self.pressure*0.25))
        end
        if moveAmount > 0 then
          sideBlock.pressure = sideBlock.pressure + moveAmount
          self.pressure = self.pressure - moveAmount
          if self.pressure <= 0 then
            level:setBlockPt(pos, blocks.new("core:sky"))      
          end
        
          level:scheduleNeighboursForUpdate(self)
          level:scheduleNeighboursForUpdate(sideBlock)
          return true
      end
      end
    end
    return false
  end
  
  
  
  --move bottom
  local bottom = pos + point.deltas.down
  local bottomBlock = level:getBlockPt(bottom)
  local movedBottom = handleSideBlock(bottomBlock, true)
  if movedBottom then
    return
  end
  
  
  --move sides
  local left = level:getBlockPt(pos + point.deltas.left)
  local right = level:getBlockPt(pos + point.deltas.right)
  
  local sideHandled = false
  sideHandled = sideHandled or handleSideBlock(left)
  sideHandled = sideHandled or handleSideBlock(right)
  if sideHandled then
    return
  end
  
  --move up
  if self.pressure > self.maxCalmPressure then
    handleSideBlock(level:getBlockPt(pos + point.deltas.up))
  end
  ]]--

end

function Water:update()
  --print("WATER UPDATE "..tostring(self.position))
  self:tick()
end

return Water