local stringutils = require("utils.stringutils")
local points = require("point")
local materials = require("material")

--local allPositions = setmetatable({}, {__mode="kv"})
local allCount = 0


local Block = {
  maxDurability = 10000,
  mass = 10,
  destructible = true,
  passable = false,
  material = materials.all.broken,
}

Block.className = "Block"
Block.isBlock = true
Block.saveKeys = {"durability"}


function Block:new()
  self.__index = self
  local instance = {
      position = points.new(),
      durability = self.maxDurability,
      level=nil
  }
  setmetatable(instance, self)
  --instance:posCheck()
  return instance
end

function Block:clone()
  local copy = self.__index:new()
  copy.position = self.position:clone()
  copy.material = self.material
  copy.durability = self.durability
  copy.destructible = self.destructible
  --self:posCheck()
  return copy
end

function Block:getDurability()
  --self:posCheck()
  return self.durability
end

function Block:setDurability(value)
  if not self.destructible then
    return
  end
  self.durability = value
  if value > self.maxDurability then
    mobdebug.on()
    local d = true
  end
  if value <= 0 then
    if self.level then
      self.level:dropBlock(self)
    else
      local debug = true
      error("dropping block which has no level!")
    end
  else
    self.level:markChunkAtBlockChanged(self)
  end
  --self:posCheck()
end


function Block:setLevel(level)
  self.level = level
  --self:posCheck()
end

function Block:getLevel()
  return self.level
end

function Block:getCanConsumeEnergy()
  local realElasticity = self.material.elasticity
  realElasticity = math.max(0.00001, math.min(realElasticity,0.99999))
  local result = self.durability / realElasticity
  return result
end

--[[function Block:posCheck()
  if true then return end
  allCount = allCount + 1
  if fmod(allCount,100000)==0 then print("check "..allCount) end
  if not self.destructible then return end
  if allPositions[self.position] == nil then
    allPositions[self.position] = self
  elseif allPositions[self.position] ~= self then
    mobdebug.on()
    local debug = true
    mobdebug.off()
  end
end]]--



function Block:getSaveData()
  --[[local result = {}
  for i, key in pairs(self.saveKeys) do
    local value = self[key]
    if type(value)=="string" then
      value = "\""..value.."\""
    end
    result[#result + 1] = key.."="..value
  end
  return table.concat(result,";")
  ]]--



  return self.durability
end

function Block:fromSaveData(data)
  --[[for savepair in saveString:gmatch("[^;]+") do
    --print("savepair="..savepair)
    for key, value in savepair:gmatch("([^=]+)=(.+)") do
      --print("key="..key.."   value="..value)
      if value:sub(1,1)~="\"" then
        value = tonumber(value)
      end
      self[key] = value
    end
  end
  ]]--
  self.durability = tonumber(data)
end


--- called when neighbour changed
function Block:update()

end


return Block
