local Queue = require("datastructures.Queue")
local Block = require("blocks.Block")
local fileutils = require("utils.fileutils")
local textureloader = require("textureloader")
local idTable = require("game.IdTable").getInstance()

local cache = {}

local blocks = {}
blocks.all = {}

function blocks.registerClass(name, class)
  blocks.all[name] = class
end

function blocks.loadBlockClass(path, modName)
  local name = fileutils.extract(path)
  print("  loading block: "..name)


  local requirepath = path.."/block"
  local blockFileExists = love.filesystem.exists(requirepath..".lua")

  local success, blockClass
  if blockFileExists then
    success, blockClass= pcall(function() return require(requirepath) end)
    if not success then
      print("block ["..name.."] crashed "..requirepath.." "..tostring(blockClass))
    end
    if success and not blockClass then
      print("block ["..name.."] script hasn't returned a value!")
      success = false;
    end
  else
    success = false
  end


  if not blockClass then
    local class = Block:new()
    blockClass = class
  end



  textureloader.loadTexturesInto(path, blockClass)
  blockClass.blockId = modName..":"..name
  if not blockClass.material or blockClass.material.name == "broken" then
    blockClass.material = blockClass.materials.texture
  end
  blocks.all[blockClass.blockId] = blockClass
  idTable:addId(blockClass.blockId)
  cache[#cache+1] = {nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,
  nil,nil,nil,nil,nil,nil,nil}
end

function blocks.rawnew(blockId)
  return blocks.all[blockId]:new()
end

blocks.new = function(blockId)
  local cl = cache[idTable:getIndex(blockId)]
  if #cl > 0 then
    local res = cl[#cl]
    cl[#cl]=nil
  end

  assert(blockId, "block Id shouldn't be nil")
  if not blocks.all[blockId] then error("no such blockId "..blockId) end
  return blocks.all[blockId]:new()
end

function blocks.release(block)
  local cl = cache[idTable:getIndex(block.blockId)]
  cl[#cl+1] = block
end


return blocks
