local M = {}
function M.trace(obj, label, padding, maxLevel, printFunction)  
  if printFunction==nil then
    printFunction = print
  end
  if not label then 
    label = ""
  end
  if not padding then
    padding = ""
  end
  if not maxLevel then
    maxLevel = 2
  end
  
  printFunction(padding.."====trace==="..label)
  if not obj then
    printFunction(padding.."  object is nil!")
    return
  end
  for k,v in pairs(obj) do
    if type(v)=="table" and maxLevel > 0 then
      M.trace(v, k, padding.."  ", maxLevel - 1, printFunction)
    else
      printFunction(padding.."  "..tostring(k).."="..tostring(v))
    end
  end  
end

M.times = {}
M.counts = {}
M.lastTime = 0
function M.addTime(name, time)
  local times = M.times
  local counts = M.counts
  if not times[name] then
    times[name] = 0
  end
  if not counts[name] then
    counts[name] = 0
  end
  times[name] = times[name] + time
  counts[name] = counts[name] + 1
end

function M.resetDelta()
  M.lastTime = love.timer.getTime()
end

function M.addDelta(name)
  M.addTime(name, love.timer.getTime() - M.lastTime)
  M.lastTime = love.timer.getTime()
end

function M.traceTime(f, message, threshold)
  local t = love.timer.getTime()
  f()
  local d = love.timer.getTime() - t
  if d >= threshold then
    print("time="..d.." message="..message)
  end  
end

return M