local Slot = require("entities.inventory.Slot")
local Stack = require("entities.inventory.Stack")
local Inventory = {}
Inventory.size = 27


function Inventory:new(size)
  self.__index = self
  local instance = {
    
    size = size
  }
  setmetatable(instance,self)
  self:initSlots()
  return instance
end

function Inventory:initSlots()
  local slots = {}
  for i = 1,self.size do
    table.insert(slots,Slot:new(i))
  end
  self.slots = slots
end

function Inventory:getSlotAt(i)
  return self.slots[i]
end

function Inventory:setSlot(i, stack)
  local validIndex = i >= 1 and i <= self.size
  
  local validEntry = stack and ( stack == EMPTY_SLOT or stack.isStack )
  if not validEntry or not validIndex then
    error("setting index "..tostring(i).."("..tostring(validIndex)..") = "..tostring(stack).."("..tostring(validEntry)..")")
  end
  self.slots[i] = stack
end

function Inventory:addEntity(entity)
  local success = self:forSlots(function(slot) 
    local stack = slot:getStack()
    if stack and stack.entity:equals(entity) then
      stack.count = stack.count + 1      
      return true
    end  
  end );
    
  if not success then 
    self:forSlots(function(slot)
      if not slot:getStack() then
        local stack = Stack:new(entity,1)
        if slot:setStack(stack)  then
          return true
        end
      end
    end);
  end
  return success
end

function Inventory:forSlots(f, filter)
  for i=1,self.size do
    local slot = self:getSlotAt(i)
    if not filter or filter(slot) then
      local result = f(slot)
      if result then return result end
    end
  end
end






return Inventory