local classUtils = require("utils.classutils")

local EventDispatcher = {}
EventDispatcher.DEFAULT_PRIORITY = 100000;


function EventDispatcher:new(parent)
  if not parent then
    parent = {}
  end
  self.__index = self
  local instance = {parent=parent}
  parent._eventDispatcher = instance
  setmetatable(instance,self)
  classUtils.copyUpFunction(parent, "_eventDispatcher","addEventListener")  
  classUtils.copyUpFunction(parent, "_eventDispatcher","removeEventListener")  
  classUtils.copyUpFunction(parent, "_eventDispatcher","dispatchEvent")  
  return instance
end

function EventDispatcher:getEventData(event)
  if not self[event] then
    self:createEventData(event)
  end
  return self[event]
end

function EventDispatcher:createEventData(event)
  self[event] = {
    strong = {},
    weak = setmetatable({},{__mode="k"})
  }
  return self[event]
end

function EventDispatcher:addEventListener(event, handler, weak, priority, once)
    assert(event, "listening to event="..tostring(event))
    assert(handler, "listening to handler="..tostring(handler))
    if weak==nil then
      weak = true
    end
    
    if priority== nil then
      priority = EventDispatcher.DEFAULT_PRIORITY
    end
    
    if once == nil then
      once = false
    end
        
    self:_addEventListener(event, handler, weak, priority, once)
end

function EventDispatcher:runOnce(event, handler)
  self:addEventListener(event,handler,false,0, true)
end

function EventDispatcher:_addEventListener(event,handler,weak, priority, once)  
  local data = self:getEventData(event)
  local refType = weak and "weak" or "strong"
  local oppositeType = weak and "strong" or "weak"
  local handlerParent = data[refType]
  local oppositeParent = data[oppositeType]
  handlerParent[handler] = {priority=priority,once=once}
  oppositeParent[handler] = nil  
  
  --print("handler added " .. tostring(self:getEventData(event)[refType][handler]))
end

function EventDispatcher:removeEventListener(event,handler)
  local data = self:getEventData(event)
  data.weak[handler] = nil
  data.strong[handler] = nil
end

function EventDispatcher:dispatchEvent(event, data)  
  local handlers = self:getEventData(event)
  local byPriority = {}
  local function addHandlersFrom(handlersTable) 
    for handler, handlerInfo in pairs(handlersTable) do
      local priority = handlerInfo.priority
      local list = byPriority[priority]
      if not list then
        --print("creating list for priority "..priority)
        list = {}
        byPriority[priority] = list
      end
      table.insert(list, {handler=handler,once=handlerInfo.once})      
      --print("inserting..."..tostring(#list))
    end     
  end
  
 
  addHandlersFrom(handlers.weak)
  addHandlersFrom(handlers.strong)
   -- for k,v in pairs(byPriority) do print("in priority "..k) end
  
  local priorities = {}
  for k,v in pairs(byPriority) do
    table.insert(priorities,k)
  end
  table.sort(priorities)
  -- print("pris:"..table.concat(priorities))
  
  for _, priority in ipairs(priorities) do
    local handlers = byPriority[priority]
    
    for j, handler in ipairs(handlers) do
     -- print("calling..."..j..tostring(handler).." priority "..priority)
      handler.handler(event, data)
      if handler.once then
        self:removeEventListener(event, handler.handler)
      end
    end
    
  end
end




return EventDispatcher