local fileutils = require("utils.fileutils")

local IdTable = {}
IdTable.FILE_NAME = "id_table.txt"

function IdTable:new()
  self.__index = self
  local instance = { idToIndex = {}, indexToId = {}}
  setmetatable(instance, self)
  return instance
end

function IdTable:addId(id)
  if self:getIndex(id) then
    return
  end
  
  local nextIndex = #self.indexToId + 1
  self.indexToId[nextIndex] = id
  self.idToIndex[id] = nextIndex
end

function IdTable:getIndex(id)
  return self.idToIndex[id]
end

function IdTable:getId(index)
  return self.indexToId[index]
end

function IdTable:getFilePath()
   return fileutils.getSavesRoot() .. self.FILE_NAME
end
function IdTable:saveToDisk()  
  love.filesystem.write(self:getFilePath(), table.concat(self.indexToId, "\n"))
end

function IdTable:loadFromDisk()
  local data = love.filesystem.read(self:getFilePath())
  if data then
    for line in data:gmatch("[^\n]+") do
      self:addId(line)
    end
  end
end

function IdTable:toThreadData()
  return self.indexToId
end

function IdTable:fromThreadData(data)
  for i, v in ipairs(data) do
    self:addId(v)
  end
end

local instance = IdTable:new()
function IdTable.getInstance() 
  return instance
end

return IdTable