local points = require("point")

local M = {}
local Renderer = {}
M.Renderer = Renderer

function Renderer:new(blockSize, screenWidth, screenHeight)
  self.__index = self
  local newObj = {
    blockSize = blockSize,
    screenWidth = screenWidth,
    screenHeight = screenHeight
    }
  return setmetatable(newObj, self)
end

function Renderer:setHud(hud)
  self.hud = hud
end


function Renderer:render(gameArea, centerPos)
  --centerPos = centerPos:round()
  self.centerPos = centerPos
  local screenHalf = points.new(self.screenWidth/2, self.screenHeight/2):round():assertInt()

  self.topLeftShift = centerPos - screenHalf --(centerPos - screeHalf):assertInt()

  for i, layer in ipairs(gameArea.layers) do
    local bright = layer == gameArea.mainLayer
    self:renderLayer(layer, i)
  end

  for i, entity in ipairs(gameArea.entities) do
    self:renderEntity(entity)
  end

  local hud = self.hud
  if hud then
    hud.size = points.new(love.graphics.getWidth(), love.graphics.getHeight())
    hud.position:set2(0,0)
    self.hud:draw()
  end


end

function Renderer:renderEntity(entity)
  local pos = entity.position - self.topLeftShift -- entity.position:round() - self.topLeftShift:assertInt()

  local topLeft = pos
  local bottomRight = topLeft + points.new(entity.shapeWidth, entity.shapeHeight)

  if bottomRight.x < 0 or bottomRight.y < 0 or topLeft.x > self.screenWidth or topLeft.y > self.screenHeight then
    -- offscreen entity - no need for rendering
    return
  end

  local tex = textures[entity.material.textureId]
  assert(tex, "no texture id "..tostring(entity.material.textureId))
  local scaleX = entity.shapeWidth / tex:getWidth()
  local scaleY = entity.shapeHeight / tex:getHeight()
  --assert(pos.x==math.floor(pos.x))
  --assert(pos.y==math.floor(pos.y))
  love.graphics.draw(tex, pos.x, pos.y, nil, scaleX, scaleY)

  if entity.equipment then
    for equipment, v in pairs(entity.equipment) do
      local targetPos = pos + equipment.shift
      local size = equipment.targetSize
      local c = {love.graphics.getColor()}
      love.graphics.setColor(60,255,60,128)
      love.graphics.rectangle("fill", targetPos.x, targetPos.y, size.x, size.y)
      love.graphics.setColor(unpack(c))
    end
  end
end


function Renderer:renderLayer(layer, renderIndex)
  local level = layer
  local centerPos = self.centerPos
  local function forBlockIndexAndPosition(center, length)
    local startPixel = center - length/2 --center - math.floor(length/2+0.5)
    local blockCount = math.ceil( length / self.blockSize)+1
    local startBlockIndex = math.floor(startPixel/self.blockSize)



    local currentIndexShift = 0
    return function()
      local i = currentIndexShift
      currentIndexShift  = currentIndexShift + 1

      if i >= blockCount then
        return nil
      else
        local blockIndex = startBlockIndex + i
        return blockIndex, blockIndex * self.blockSize - startPixel
      end
    end
  end

  local blockCounter = 0
  local tempColor = {love.graphics.getColor()}

  for xI, xPos in forBlockIndexAndPosition(centerPos.x, self.screenWidth) do
    for yI, yPos in forBlockIndexAndPosition(centerPos.y, self.screenHeight) do
      if xPos == math.floor(xPos) then print("xPos is int!"); else print("xPos is float!") end
      local block = level:getBlock(xI,yI)
--      if not block.material then
--        print("faulty");
--        for k,v in pairs(block) do print(k.."="..tostring(v)) end
--      end
--      assert(block.material, "no material id="..tostring(block.__index.blockId).." pos="..tostring(block.position))
      local texture = textures[block.material.textureId];
      --assert(texture, "no texture with id "..tostring(block.material.textureId))

      if not texture then
        texture = textures["broken"]
        --print("no texture with id "..tostring(block.material.textureId))
      end
      local durabilityRatio = block.durability / block.maxDurability

      if durabilityRatio > 0 then
        local isSky = block.blockId == "core:sky"
        local showSky = renderIndex == 1 and isSky
        local bright = layer.index == 0
        local brightness = (showSky) and 1 or (math.pow(0.6, layer.index))
        local alpha = (isSky and not showSky) and 0 or 255

        local fullColor = brightness * 255
        local durabilityColor = fullColor * durabilityRatio

        love.graphics.setColor(fullColor, durabilityColor, durabilityColor,alpha)
      else
        love.graphics.setColor(64,0,0)
      end
      love.graphics.draw(texture, xPos, yPos)
      --[[if block.pressure then
        local oldFont = love.graphics.getFont()
        love.graphics.setFont(font7)
        love.graphics.setColor(0,0,0)
        love.graphics.print(string.sub((block.pressure),0,3),xPos+4,yPos+4)
        love.graphics.setFont(oldFont)
      end]]--


      blockCounter = blockCounter + 1
    end
  end
  love.graphics.setColor(unpack(tempColor))
  self.blocksRendered = blockCounter
end


return M
