-- GLOBALS: isDebug, ___lockGlobals, ___lockUndefinedRead, ___unlockGlobals, ___modifyGlobals
pcall(function()
  mobdebug = require("mobdebug")
  isDebug = arg[#arg] == "-debug"
  if  isDebug then mobdebug.start() end
end)



-- LOCK GLOBALS TO PREVENT ERRORS

function ___lockUndefinedRead(_table)
  setmetatable(_table, {__index=function(t, k) error("access of undefined global "..tostring(k)) end })
end



function ___lockGlobals()
  local globalsMeta = getmetatable(_G)
  globalsMeta.__newindex = function(t,k,v) error("trying to set global "..tostring(k).."="..tostring(v)) end
end

function ___unlockGlobals()
  local globalsMeta = getmetatable(_G)
  globalsMeta.__newindex = function(t,k,v) print("global set "..tostring(k).."="..tostring(v)); rawset(t,k,v) end
end

function ___modifyGlobals(handler)
  ___unlockGlobals()
  handler()
  ___lockGlobals()
end

___lockUndefinedRead(_G)
___lockGlobals()
--END LOCKING GLOBALS






local testRunFunction = love.filesystem.load("test/test.lua")
testRunFunction(false)
pcall( function() mobdebug.off() end)



local idTable = require("game.IdTable").getInstance()
local BlockLayer = require("game.BlockLayer")
local Entity = require("entities.Entity")
local BasicTool = require("entities.tools.BasicTool")
local keyboardControllers = require("userinput.keyboardcontroller")
local ToolController = require("userinput.ToolController")
local points =require("point")
local materials = require("material")
local renderers = require("graphics.rendererTemp")--require("graphics.renderer")
local modloaders = require("modloader")
local blocks = require("blocks.blocks")
local Block = require("blocks.Block")
local TurnRunner = require("time.turnrunner")
local turnRunner = TurnRunner.default
local BlockEntity = require("entities.BlockEntity")
local debugUtil = require("debug.debugUtil")
local entityBuilding = require("entities.entityBuilding")
local Hud = require("graphics.Hud")
local Chunk = require("game.Chunk")
local EventDispatcher = require("events.EventDispatcher")
local threadCallbacks = require("threads.ThreadCallBacks").getInstance()


___modifyGlobals(
  function()
    -- GLOBALS: BLOCK_SIZE, dispatcher, textures, GameGlobals, fmod, font7, err
    BLOCK_SIZE = 16
    dispatcher = EventDispatcher:new()
    textures = {}
    GameGlobals = {
      title = "Crafting platformer #81BD9D2A-71E4-11E7-8EC3-79DD75302EFD"
    }
    setmetatable(_G, {__index=function(t, k) error("access of undefined global "..tostring(k)) end })
    function fmod(a,b)
      local r = math.fmod(a,b)
      return r < 0 and r+b or r
    end
    font7 = love.graphics.newFont(7)
    --no printing for you
    --print = function() end
    local errHandle = io.stdout
    function err(msg)
      errHandle:write(msg.."\n")
      errHandle:flush()
    end
  end
)

local GameArea = require("game.GameArea")

local gameArea = nil



local entity = Entity:new()
local met1 = getmetatable(entity)

entity.position.x = 130--50
entity.position.y = 3744 -- BLOCK_SIZE*30
entity.playerControlled = true
entityBuilding.makeWithEquipment(entity)
entityBuilding.makeWithInventory(entity)
entity.inventory:addEntity(entity)


--entity.mass = 100000

local tool = BasicTool:new()
entity:equip(tool)



local level
local background



local function tex(name)
    textures[name] = love.graphics.newImage("/assets/"..name..".png")
    return textures[name]
end

local function getTurnTime()
  return math.floor(love.timer.getTime()*100)
end

function love.load()
  testRunFunction(true)
  
  love.graphics.setDefaultFilter("linear", "linear", 16)
  love.window.setTitle(GameGlobals.title)
  love.window.setMode(math.floor(1920*0.8), math.floor( (1080-100)*0.95), {resizable=true, vsync=true})
  idTable:loadFromDisk()
  gameArea = GameArea:new()
  turnRunner:setTime(getTurnTime())

  local _keyController = keyboardControllers.KeyboardController:new(entity, points.new(0,0), points.new(gameArea.LEVEL_WIDTH*BLOCK_SIZE, gameArea.LEVEL_HEIGHT*BLOCK_SIZE))
  local _renderer = renderers.Renderer:new(BLOCK_SIZE, love.graphics.getWidth(), love.graphics.getHeight())

  local _modLoader = modloaders.ModLoader:new()
  local _toolController = ToolController:new(tool, gameArea)
  GameGlobals.hud = Hud:new()

  GameGlobals.hud:setEntity(entity)
  _renderer:setHud(GameGlobals.hud)

  ___modifyGlobals(function()
      -- GLOBALS: keyController, renderer, modLoader, toolController, focusControl
      keyController = _keyController
      renderer = _renderer
      modLoader = _modLoader
      toolController = _toolController
      focusControl = GameGlobals.hud
  end)

  modLoader:load()
  idTable:saveToDisk()

  level = BlockLayer:new(gameArea.LEVEL_WIDTH,gameArea.LEVEL_HEIGHT, 0)
  local bgLevel = BlockLayer:new(gameArea.LEVEL_WIDTH, gameArea.LEVEL_HEIGHT, 29305783, 1)
  local bgLevel2 = BlockLayer:new(gameArea.LEVEL_WIDTH, gameArea.LEVEL_HEIGHT, 2034, 2)
  table.insert(level.chunkLoaders, entity)
  table.insert(bgLevel.chunkLoaders,entity)
  table.insert(bgLevel2.chunkLoaders, entity)

  for k, v in pairs(blocks.all) do
    print("block list: "..k)
  end

  background = tex("img1")
  tex("flying_medusa")
  tex("broken")

  GameGlobals.defaultShader = love.graphics.getShader()

  local shader = love.graphics.newShader[[
      vec4 effect(vec4 color, Image texture, vec2 coords, vec2 screen_coords){
        vec4 o = Texel(texture, coords);
        number x = coords.x;
        number y = coords.y;
        number dist = abs(x-0.5)+abs(y-0.5);
        number brightness = (1.5- dist )/ 1.5;
        o.r = (o.r + 0.1)*brightness;
        o.g = (o.g + 0.1)*brightness;
        o.b = (o.b + 0.1)*brightness;
        o.a = o.a < 0.3 ? 0.3 : o.a;
        return o;
      }
  ]]

  gameArea:addLayer(bgLevel2)
  gameArea:addLayer(bgLevel)
  gameArea:addLayer(level, true)
  gameArea:addEntity(entity)

  local earth = blocks.rawnew("core:earth")
  local earthEntity= BlockEntity:new(earth)
  local earth2 = blocks.rawnew("core:earth")
  local earthEntity2= BlockEntity:new(earth2)
  entity.inventory:addEntity(earthEntity)
  for i = 1,100 do
    entity.inventory:addEntity(earthEntity2)
  end


end

local function drawBackground()
  love.graphics.setColor(160,232,237)
  local w = love.graphics.getWidth()
  local h = love.graphics.getHeight()
  local wRatio = w / background:getWidth()
  local hRatio = h / background:getHeight()
  --love.graphics.draw(background, 0,0, 0, wRatio, hRatio)
  love.graphics.rectangle("fill",0,0,w,h)
end

local lastTick = nil
local maxDt = 0.05
local maxMeasuredDt = 0
local maxDtTimeout = 1000
local paused = false
local manualSteps = 0


local deltaTimes = {}
local currentDeltaIndex = 1

function love.update(dt)

  deltaTimes[currentDeltaIndex] = dt
  currentDeltaIndex = currentDeltaIndex+1
  if currentDeltaIndex > love.graphics.getWidth() then
    currentDeltaIndex = 1
  end
  threadCallbacks:handleAsyncResults(1/240)

  if paused  then
    if  manualSteps > 0 then
      manualSteps = manualSteps - 1
    else
      return
    end
  end

  if dt > maxMeasuredDt then
    maxMeasuredDt = dt
    maxDtTimeout = 1000
  else
    maxDtTimeout = maxDtTimeout - 1
    if maxDtTimeout <= 0 then
      maxMeasuredDt = 0
    end
  end

  if dt < 1/240 then
    return
  end
  if dt > maxDt then
    dt = maxDt
  end
  --
  -- if dt > maxDt then
  --   print("DT="..dt)
  --   dt = maxDt
  -- end

  dispatcher:dispatchEvent("enterFrame")
  if not lastTick then
    lastTick = love.timer.getTime()
  end

  if love.timer.getTime() - lastTick > 1 then
    lastTick = love.timer.getTime()
    dispatcher:dispatchEvent("tick", {level=level})
  else

  end

  if dt then
    keyController:step(dt)
    toolController:update(dt)

  end


  turnRunner:stepToTime(getTurnTime())

  gameArea:handleDt(dt)
end
local drawTest;

local function getScreenPrinter()
  local lineHeight = 16
  local startY = 40
  local lineIndex = 0

  local printer = function(thing)
    love.graphics.setColor(0,64,128)
    love.graphics.print(tostring(thing), 10, startY+ lineIndex * lineHeight)
    lineIndex = lineIndex + 1
  end
  return printer
end





function love.draw()
  drawBackground()
  love.graphics.setBackgroundColor(0,0,0)
  love.graphics.setColor(0,0,0)
  love.graphics.print("Hello world!", 10,10)
  love.graphics.setColor(255,255,255)

  local w = love.graphics.getWidth();
  local h = love.graphics.getHeight();



  love.graphics.setColor(255,255,255,255)
  renderer:render(gameArea, entity.position)



  if paused then
    love.graphics.setColor(255,255,255,180)
    love.graphics.rectangle("fill", w/2-32, h/2-8,64,16)
    love.graphics.setColor(0,0,0,255)

    love.graphics.print("PAUSE", w/2-20, h/2-6)
  end



  local printDebugInfo = false
  if printDebugInfo then
    local stats = love.graphics.getStats()
    stats._FPS = ""..love.timer.getFPS()
    stats._Blocks = ""..tostring(renderer.blocksRendered).."/"..(level.width * level.height)
    stats._AtlasUpdated = tostring(renderer.atlasUpdated)
    stats._Entities = ""..tostring(#gameArea.entities)
    stats._EntityTimes = tostring(gameArea.timesFormatted)
    stats._ChunkTimes = Chunk.getStats()
    stats._MaxDT = maxMeasuredDt




    --drawTest()
    local printer = getScreenPrinter()
    debugUtil.trace(stats, "stats", nil, nil, printer)
    printer("")
    debugUtil.trace(debugUtil.times, "times", nil, nil, printer)
    printer("")
    debugUtil.trace(debugUtil.counts, "counts", nil, nil, printer)
    printer("")
    local entityStats = {speed=entity.speed, acceleration=entity.acceleration, position=entity.position}
    debugUtil.trace(entityStats, "entity", nil, nil, printer)
  end



  local statsH = 200
  local maxExpecteDt = 0.1
  for i=1, #deltaTimes do
    local value = statsH * deltaTimes[i]/maxExpecteDt
    if value > statsH then
      love.graphics.setColor(255,128,0,255)
    elseif i+1 == currentDeltaIndex then
      love.graphics.setColor(255,0,0,255)
    else
      local age = currentDeltaIndex-i
      if age < 0 then
        age = age+love.graphics.getWidth()
      end
      love.graphics.setColor(math.max(0,math.min(255,2*value)),128,0,255-255*(age/love.graphics.getWidth()))
    end
    love.graphics.rectangle("fill",i, 0, 1, value)
    love.graphics.setColor(255,255,255,255)
  end

  love.graphics.print(love.timer.getFPS().."    x speed="..math.floor(entity.speed.x))

end


function love.keypressed(key,scancode,isrepeat)

  if key == "pause" then
    paused = not paused
  end
  if key == "space" then
    manualSteps = manualSteps + 1
  end

  focusControl:keypressed(key,scancode,isrepeat)
end





local testData
local testImg
 function drawTest()

  local Noise = require("generation.noise.Noise")
  local noiseR = Noise:new()
  local noiseG = Noise:new(29)
  local noiseB = Noise:new(1305097)

  local function toColor(noiseValue)
    return noiseValue * 128 + 128
  end

  if not testData then
    testData = love.image.newImageData(love.graphics.getWidth(), love.graphics.getHeight())
    for x=1, love.graphics.getWidth() do
      print(x)
      --io.flush()
      for y=1, love.graphics.getHeight() do
        local r=0
        local g=0
        local b=0

        local mins=1
        local maxs=8
       -- local xm = x-500
       -- local ym = y-500
        for s=mins,maxs do
          local scale = math.pow(2,s)
          local antiscale = math.pow(2,maxs+1-s)
          local sx = x/scale
          local sy = y/scale
          r = r + noiseR:get2d(sx,sy)/antiscale
         -- g = g + noiseG:getNd(sx,sy)
          --b = b + noiseB:getNd(sx,sy)
        end
        testData:setPixel(x-1,y-1, toColor(r),0,0,255)
      end
    end
  end
  if not testImg then
    testImg = love.graphics.newImage(testData)
  end
  love.graphics.setColor(255,255,255,255)
  love.graphics.draw(testImg)



end
