local points = require("point")
local Point = points.Point
local LiquidHelper = {}
function LiquidHelper:new(layer)
  self.__index = self
  local instance = { layer=layer}
  setmetatable(instance, self)
  return instance
end

function LiquidHelper:_addConnectedBlocks(block)
  if not block or not block.isLiquid then
    error("Attempt to add a liquid area starting from non-liquid block"..(block and tostring(block.blockId) or "nil"))
  end
  local result = {}
  
  local layer = self.layer
  
  local queue = {[1]=block}
  local used = { [block]=true}
  local qStart = 1  
  while qStart <= #queue do
    local current = queue[qStart]
    qStart = qStart + 1
    for _, delta in pairs(points.deltas) do
      local neighbour = layer:getBlockPt(current.position + delta)
      if neighbour.isLiquid and not used[neighbour] then--here
        table.insert(result, neighbour)
        used[neighbour] = true
        table.insert(queue, neighbour)
      end
    end
  end
  table.sort(result, function(a,b) return a.position.y < b.position.y end)
  return  result
end

function LiquidHelper:_tryReplaceNeighbours(block, getReplacements)  
  local replacements = nil
  local firstUnusedBlockIndex = 1
  local maxUnusedBlockIndex = nil
  --print("replacements: "..tostring(maxUnusedBlockIndex))
  for _, delta in pairs(points.deltas) do    
    local neighbour = self.layer:getBlockPt(block.position + delta)
    if neighbour.isReplacible then
      if not replacements then
        replacements = getReplacements()
        maxUnusedBlockIndex = #replacements
      end
      if firstUnusedBlockIndex <= maxUnusedBlockIndex then        
        local replacement = replacements[firstUnusedBlockIndex]
        if neighbour.isReplacible and replacement.position.y < neighbour.position.y then
          local temp = neighbour.position:clone()      
          self.layer:setBlockPt(replacement.position, neighbour)
          self.layer:setBlockPt(temp, replacement)
          firstUnusedBlockIndex = firstUnusedBlockIndex+1
        end
      end
    end
    
  end  
end

function LiquidHelper:handleBlock(block)  
  self:_tryReplaceNeighbours(block, function() return  self:_addConnectedBlocks(block) end)
end

return LiquidHelper
