local SurfaceResistance = {}

function SurfaceResistance:new( layer)
  assert(layer)
  self.__index = self
  local instance = {layer=layer}
  setmetatable(instance,self)
  return instance
end


function SurfaceResistance:handle(entity)
  local size = entity:getSize()
  local function handleDirection(coord)
    if math.abs(entity.speed[coord]) < 5 then
      local signThing = entity.acceleration[coord]
      if signThing == 0 then
        --predict speed based on acceleration
        signThing = entity.speed[coord]
      end
      if signThing == 0 then
        --no speed, no acceleration -> nothing to do
        return true
      end
      local sign = signThing > 0 and 1 or -1
      local blockPos = entity.position:clone()
      blockPos[coord] = blockPos[coord] + sign * BLOCK_SIZE
      local block = self.layer:getBlockPx(blockPos)
      if not block.passable then
        entity.speed[coord] = 0
        entity.position[coord] = sign > 0 and block.position[coord]*BLOCK_SIZE-size[coord] or (block.position[coord]+1)*BLOCK_SIZE
        entity.acceleration[coord] = 0
        return true
      end
    end
  end
  local stopped = handleDirection("x")
  stopped = stopped and handleDirection("y")
  return stopped
end

return SurfaceResistance
