local ThreadCallBacks = {}
function ThreadCallBacks:new()
  self.__index = self
  local instance = {
    resultChannel = love.thread.getChannel("results"),
    callbackCounter = 1,
    callbacks = {}
  }

  setmetatable(instance, self)
  return instance
end

function ThreadCallBacks:registerCallBack(callBack)
  local callBackIndex = self.callbackCounter
  self.callbackCounter = self.callbackCounter + 1
  self.callbacks[callBackIndex] = callBack
  return callBackIndex
end


function ThreadCallBacks:handleAsyncResults(maxTime)
  if not maxTime then maxTime = 1 end
  local startTime = love.timer.getTime()
  local resultChannel = self.resultChannel
  local callbacks = self.callbacks
  repeat
    local message = resultChannel:pop()
    if message then

      local callBackIndex,data,data2 = unpack(message)
      local callBack = callbacks[callBackIndex]
      if callBack  then
        callBack(data,data2)
        callbacks[callBackIndex] = nil
        if love.timer.getTime()  - startTime > maxTime then
          break
        end
      else
        err("no callback with index ".. tostring(callBackIndex))
      end
    end
  until not message
end

local instance = ThreadCallBacks:new()
function ThreadCallBacks.getInstance()
  return instance
end

return ThreadCallBacks
