

local ffi = require("ffi")
ffi.cdef[[
  typedef struct {
    char *fpos;
    void *base;
    unsigned short handle;
    short flags;
    short unget;
    unsigned long alloc;
    unsigned short buffincrement;
  } FILE;

  FILE *fopen(const char *filename, const char *mode);
  int fprintf(FILE *stream, const char *format, ...);
  int fclose(FILE *stream);
  size_t fwrite(const void *ptr , size_t size, size_t nmemb, FILE *stream);
  int fseek(FILE *stream, long int offset, int whence);
  long int ftell(FILE *stream);
  size_t fread(void *ptr, size_t size, size_t nmemb, FILE *stream);

  void *malloc(size_t size);
  void free(void* memblock);
]]

local SEEK_END = 2
local SEEK_CUR = 1
local SEEK_SET = 0

local C = ffi.C

local args = {...}

if type(args[1])~="string" then
  error("pass first arg=input channel name with thread.start()")
end
local outputChannelName  = args[2] and args[2] or "results"
local inputChannel = love.thread.getChannel(args[1])
local outputChannel = love.thread.getChannel(outputChannelName)

repeat
  local args = inputChannel:demand()

  local operation = args[1]
  local callBackIndex = args[#args]

  local result = nil
  local result2 = nil
  if operation == "open" then
    local fileName = args[2]
    local f = C.fopen(fileName,"rb+")
    --print(tostring(f));
    if f==nil then
      --print("nil")
      f = C.fopen(fileName,"wb+")
      --print(tostring(f));
      --if f==nil then
        --print("filename="..tostring(fileName).." error="..tostring(ffi.errno()))
      --end
    end
    io.stdout:flush()
    result = tonumber(ffi.cast("intptr_t", f))

  elseif operation == "close" then
    local filePtr = ffi.cast("FILE *", args[2])
    C.fclose(filePtr)
  elseif operation == "write" then
    local filePtr = ffi.cast("FILE *",args[2])
    local offset = args[3]
    local dataPtr = ffi.cast("char *",args[4])
    local dataSize = args[5]

    local seekError
     result = offset
    if offset ==-1 then
      seekError = C.fseek(filePtr, 0, SEEK_END)
      result = C.ftell(filePtr)
    else
      seekError = C.fseek(filePtr, offset, SEEK_SET)
    end
    if seekError ~= 0 then
      local message = "file seek error. "..tostring(seekError).." offset="..tostring(offset).. " code "..tostring(ffi.errno().." file "..tostring(filePtr).." result="..tostring(result).. " datasize="..tostring(dataSize))
      print(message); io.stdout:flush()
      error(message)
    else
      C.fwrite(dataPtr, 1, dataSize, filePtr)
    end
    C.free(dataPtr)
  elseif operation == "read" then
    local filePtr = ffi.cast("FILE *",args[2])
    local offset = args[3]
    local dataSize = args[4]

    local dataPtr = C.malloc(dataSize)

    local seekResult = C.fseek(filePtr, offset, SEEK_SET)

    local numberRead = C.fread(dataPtr, 1, dataSize, filePtr)
    result = tonumber(ffi.cast("intptr_t",dataPtr))
    if numberRead~=dataSize then
      result2 =tonumber(numberRead)
    end
  else
    local message = "RW FILE thread stopping: invalid operation "..tostring(operation)
    print(message)
    io.stdout:flush()
    error(message)
  end


  outputChannel:push({callBackIndex, result, result2})
until false
