local ffi = require("ffi")

ffi.cdef[[
  typedef struct {
    char *fpos;
    void *base;
    unsigned short handle;
    short flags;
    short unget;
    unsigned long alloc;
    unsigned short buffincrement;
  } FILE;

  FILE *fopen(const char *filename, const char *mode);
  int fprintf(FILE *stream, const char *format, ...);
  int fclose(FILE *stream);
  size_t fwrite(const void *ptr , size_t size, size_t nmemb, FILE *stream);

  void free(void* memblock);
]]

local C = ffi.C



local inputChannel = love.thread.getChannel("saveFileChannelParams")
local outputChannel = love.thread.getChannel("results")

repeat
  local args = inputChannel:demand()
  local fileName, data, dataSize, callBackIndex
  local dataPtr
  if type(data)=="string" then
     fileName, data, callBackIndex = unpack(args)
     dataSize = #data
  else
    fileName, dataPtr, dataSize, callBackIndex = unpack(args)
    io.stdout:flush()
    data = ffi.cast("char *", ffi.cast("intptr_t", dataPtr))
    local data2 = ffi.cast("double *", data)
    for i=0, (dataSize/ ffi.sizeof("double"))-1 do
      if i%2 == 0 and data2[i] ~= math.floor(data2[i]) then
         error("save data is already corrupted!"..tonumber(dataPtr))
      end
    end
  end
  local f = C.fopen(fileName, "w")
  C.fwrite(data, 1, dataSize,f)
  C.fclose(f)


  C.free(data)
  outputChannel:push({callBackIndex})
until false
