local TurnRunner = {}
function TurnRunner:new()
  self.__index = self
  local instance = {
    times = {},
    currentTime = 0
  }
  return setmetatable(instance, self)
end

function TurnRunner:setTime(time)
  self.currentTime = time
end

function TurnRunner:getHandlersFor(time)
  local times = self.times
  if not times[time] then
    times[time] = setmetatable({}, {__mode="v"})
  end
  return times[time]
end

function TurnRunner:addTimeListener(time, handler)
  table.insert(self:getHandlersFor(time), handler)
  --print("turn handler added "..time)
end

function TurnRunner:scheduleListenerIn(interval, handler)
  self:addTimeListener(self.currentTime+interval, handler)
end

function TurnRunner:runHandlersFor(time)
  
  local handlers = self:getHandlersFor(time)
  --print("turn runner: I'm running "..(#handlers).."handlers for "..time)
  for _, handler in pairs(handlers) do
    handler()
  end
end

function TurnRunner:getMinTime() 
  local minTime = nil
  for k,v in pairs(self.times) do
    if not minTime or k < minTime then
      minTime = k
    end
  end
  return minTime
end


function TurnRunner:step()
  local minTime = self:getMinTime()
  if not minTime then
    return
  end
  self.currentTime = minTime
  self:runHandlersFor(minTime)
  self.times[minTime] = nil
  
end

function TurnRunner:stepToTime(time)  
  local handled
  repeat
    local minTime = self:getMinTime()
    if minTime and minTime <= time then
      self:step()
    else
      if not minTime then
          self.currentTime = time
      else 
        -- print(""..minTime..">"..time)
      end
      handled = true
    end
  until handled    
end

TurnRunner.default = TurnRunner:new()

return TurnRunner