local points = require("point")
local M = {}
local KeyboardController = {}
M.KeyboardController = KeyboardController

 local config =  nil
function M.initConfig()
  config = {}
  for k,v in pairs(love.keyboard) do
   -- print ("key="..k)
  end
  if not love.keyboard.KeyConstant then
    love.keyboard.KeyConstant = { up="up", down="down", left="left",right="right",w="w",a="a",s="s",d="d"}
  end
  config[love.keyboard.KeyConstant.up] = "up"
  config[love.keyboard.KeyConstant.down] = "down"
  config[love.keyboard.KeyConstant.left] = "left"
  config[love.keyboard.KeyConstant.right] = "right"

  config[love.keyboard.KeyConstant.w] = "up"
  config[love.keyboard.KeyConstant.s] = "down"
  config[love.keyboard.KeyConstant.a] = "left"
  config[love.keyboard.KeyConstant.d] = "right"
end

function KeyboardController:new(thingWithPosition)
  if not thingWithPosition then
    error("parameters must not be null")
  end
  self.__index = self
  local instance = {
    objectToControl = thingWithPosition,
    minAcceleration = 5,
    maxAcceleration = 30,
    increaseToMaxDelay = 0.5,
    keyDurations = {}
  }
  return setmetatable(instance,self)  
end

function KeyboardController:step(dt)
  if not dt then
    error("dt shouldn't be nil")
  end
  if not config then
    M.initConfig()
  end
  
  local sumDelta = points.new()
  for key,action in pairs(config) do
    if love.keyboard.isDown(key) then
      if not self.keyDurations[key] then 
        self.keyDurations[key] = dt*0.5 
      else 
        self.keyDurations[key] = self.keyDurations[key] + dt; 
      end
      
      local duration = self.keyDurations[key]
      local possibleIncrease = self.maxAcceleration - self.minAcceleration
      local realIncrease = possibleIncrease * duration / self.increaseToMaxDelay
      local acceleration = self.minAcceleration + realIncrease
      
      local delta = points.deltas[action] 
      if delta then
        sumDelta  = sumDelta + delta * acceleration
      end    
    else
      self.keyDurations[key] = 0
    end
    
  end
  self.objectToControl:addForce(sumDelta * self.objectToControl.mass)
end



return M